@@ -12,7 +12,7 @@ pub mod bank;
 pub mod bank_client;
 pub mod bank_forks;
 pub mod bank_utils;
-mod blockhash_queue;
+pub mod blockhash_queue;
 pub mod bloom;
 pub mod builtins;
 pub mod commitment;
@@ -36,6 +36,7 @@ pub mod serde_snapshot;
 pub mod snapshot_package;
 pub mod snapshot_utils;
 pub mod sorted_storages;
+pub mod stake_weighted_timestamp;
 pub mod stakes;
 pub mod status_cache;
 mod system_instruction_processor;
